import 'package:ambito/src/domain/entity/baumarten/baumarten.dart';
import 'package:ambito/src/domain/entity/massnahme/massnahme.dart';
import 'package:ambito/src/packages/ambito_api/base.dart';
import 'package:ambito/src/pages/start/start_page.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_i18n/flutter_i18n.dart';
import 'package:get/get_navigation/src/root/get_material_app.dart';
import 'package:isar/isar.dart';
import 'package:logger/logger.dart';

const String ambitoToken = 'TFxO7vzBLVRCu9I3VMoHmTuCvSu8aCDi';

final Logger logger = Logger(
  printer: PrettyPrinter(),
);

late Isar isar;

void main() async {
  WidgetsFlutterBinding.ensureInitialized();

  await Isar.initialize();
  const dir = Isar.sqliteInMemory;
  const engine = kIsWeb ? IsarEngine.sqlite : IsarEngine.isar;
  isar = Isar.open(
    schemas: [BaumartenSchema, MassnahmeSchema],
    directory: dir,
    engine: engine,
    inspector: true,
  );

  Future.wait([
    BaseApi().getContent('baumarten'),
    BaseApi().getContent('massnahmen'),
  ]);

  //await BaseApi().getContent('baumarten');
  //await BaseApi().getContent('massnahmen');

  runApp(const Ambito());
}

class Ambito extends StatelessWidget {
  const Ambito({super.key});
  @override
  Widget build(BuildContext context) {
    return GetMaterialApp(
      debugShowCheckedModeBanner: false,
      localizationsDelegates: [
        FlutterI18nDelegate(
          translationLoader: FileTranslationLoader(
            fallbackFile: 'de',
            basePath: 'i18n',
          ),
          missingTranslationHandler: (key, locale) {
            if (kDebugMode) {
              print("--- Missing Key: $key, languageCode: $locale");
            }
          },
        ),
      ],
      title: 'Flutter Demo',
      locale: const Locale('de'),
      builder: FlutterI18n.rootAppBuilder(),
      theme: ThemeData(
        scaffoldBackgroundColor: Colors.white,
        useMaterial3: true,
      ),
      home: const StartPage(),
    );
  }
}

extension ContextI18n on BuildContext {
  translate(String key) {
    return FlutterI18n.translate(this, key);
  }
}
